@@ -12,10 +12,12 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import io
 import json
 import unittest
 
 import mock
+import pytest
 import requests
 from six.moves import http_client
 
@@ -60,51 +62,71 @@ class TestClient(unittest.TestCase):
         return self._get_target_class()(*args, **kw)
 
     def test_ctor_connection_type(self):
+        from google.cloud._http import ClientInfo
         from google.cloud.storage._http import Connection
 
         PROJECT = "PROJECT"
-        CREDENTIALS = _make_credentials()
+        credentials = _make_credentials()
 
-        client = self._make_one(project=PROJECT, credentials=CREDENTIALS)
+        client = self._make_one(project=PROJECT, credentials=credentials)
 
         self.assertEqual(client.project, PROJECT)
         self.assertIsInstance(client._connection, Connection)
-        self.assertIs(client._connection.credentials, CREDENTIALS)
+        self.assertIs(client._connection.credentials, credentials)
         self.assertIsNone(client.current_batch)
         self.assertEqual(list(client._batch_stack), [])
+        self.assertIsInstance(client._connection._client_info, ClientInfo)
 
     def test_ctor_wo_project(self):
         from google.cloud.storage._http import Connection
 
         PROJECT = "PROJECT"
-        CREDENTIALS = _make_credentials()
+        credentials = _make_credentials()
 
         ddp_patch = mock.patch(
             "google.cloud.client._determine_default_project", return_value=PROJECT
         )
 
         with ddp_patch:
-            client = self._make_one(credentials=CREDENTIALS)
+            client = self._make_one(credentials=credentials)
 
         self.assertEqual(client.project, PROJECT)
         self.assertIsInstance(client._connection, Connection)
-        self.assertIs(client._connection.credentials, CREDENTIALS)
+        self.assertIs(client._connection.credentials, credentials)
         self.assertIsNone(client.current_batch)
         self.assertEqual(list(client._batch_stack), [])
 
     def test_ctor_w_project_explicit_none(self):
         from google.cloud.storage._http import Connection
 
-        CREDENTIALS = _make_credentials()
+        credentials = _make_credentials()
 
-        client = self._make_one(project=None, credentials=CREDENTIALS)
+        client = self._make_one(project=None, credentials=credentials)
 
         self.assertIsNone(client.project)
         self.assertIsInstance(client._connection, Connection)
-        self.assertIs(client._connection.credentials, CREDENTIALS)
+        self.assertIs(client._connection.credentials, credentials)
         self.assertIsNone(client.current_batch)
         self.assertEqual(list(client._batch_stack), [])
 
+    def test_ctor_w_client_info(self):
+        from google.cloud._http import ClientInfo
+        from google.cloud.storage._http import Connection
+
+        credentials = _make_credentials()
+        client_info = ClientInfo()
+
+        client = self._make_one(
+            project=None, credentials=credentials, client_info=client_info
+        )
+
+        self.assertIsNone(client.project)
+        self.assertIsInstance(client._connection, Connection)
+        self.assertIs(client._connection.credentials, credentials)
+        self.assertIsNone(client.current_batch)
+        self.assertEqual(list(client._batch_stack), [])
+        self.assertIs(client._connection._client_info, client_info)
+
     def test_create_anonymous_client(self):
         from google.auth.credentials import AnonymousCredentials
         from google.cloud.storage._http import Connection
@@ -261,7 +283,7 @@ class TestClient(unittest.TestCase):
         self.assertIsInstance(batch, Batch)
         self.assertIs(batch._client, client)
 
-    def test_get_bucket_miss(self):
+    def test_get_bucket_with_string_miss(self):
         from google.cloud.exceptions import NotFound
 
         PROJECT = "PROJECT"
@@ -290,7 +312,7 @@ class TestClient(unittest.TestCase):
             method="GET", url=URI, data=mock.ANY, headers=mock.ANY
         )
 
-    def test_get_bucket_hit(self):
+    def test_get_bucket_with_string_hit(self):
         from google.cloud.storage.bucket import Bucket
 
         PROJECT = "PROJECT"
@@ -320,6 +342,68 @@ class TestClient(unittest.TestCase):
             method="GET", url=URI, data=mock.ANY, headers=mock.ANY
         )
 
+    def test_get_bucket_with_object_miss(self):
+        from google.cloud.exceptions import NotFound
+        from google.cloud.storage.bucket import Bucket
+
+        project = "PROJECT"
+        credentials = _make_credentials()
+        client = self._make_one(project=project, credentials=credentials)
+
+        nonesuch = "nonesuch"
+        bucket_obj = Bucket(client, nonesuch)
+        URI = "/".join(
+            [
+                client._connection.API_BASE_URL,
+                "storage",
+                client._connection.API_VERSION,
+                "b",
+                "nonesuch?projection=noAcl",
+            ]
+        )
+        http = _make_requests_session(
+            [_make_json_response({}, status=http_client.NOT_FOUND)]
+        )
+        client._http_internal = http
+
+        with self.assertRaises(NotFound):
+            client.get_bucket(bucket_obj)
+
+        http.request.assert_called_once_with(
+            method="GET", url=URI, data=mock.ANY, headers=mock.ANY
+        )
+
+    def test_get_bucket_with_object_hit(self):
+        from google.cloud.storage.bucket import Bucket
+
+        project = "PROJECT"
+        credentials = _make_credentials()
+        client = self._make_one(project=project, credentials=credentials)
+
+        bucket_name = "bucket-name"
+        bucket_obj = Bucket(client, bucket_name)
+        URI = "/".join(
+            [
+                client._connection.API_BASE_URL,
+                "storage",
+                client._connection.API_VERSION,
+                "b",
+                "%s?projection=noAcl" % (bucket_name,),
+            ]
+        )
+
+        data = {"name": bucket_name}
+        http = _make_requests_session([_make_json_response(data)])
+        client._http_internal = http
+
+        bucket = client.get_bucket(bucket_obj)
+
+        self.assertIsInstance(bucket, Bucket)
+        self.assertEqual(bucket.name, bucket_name)
+        http.request.assert_called_once_with(
+            method="GET", url=URI, data=mock.ANY, headers=mock.ANY
+        )
+
     def test_lookup_bucket_miss(self):
         PROJECT = "PROJECT"
         CREDENTIALS = _make_credentials()
@@ -376,70 +460,184 @@ class TestClient(unittest.TestCase):
             method="GET", url=URI, data=mock.ANY, headers=mock.ANY
         )
 
-    def test_create_bucket_conflict(self):
+    def test_create_bucket_with_string_conflict(self):
         from google.cloud.exceptions import Conflict
 
-        PROJECT = "PROJECT"
-        OTHER_PROJECT = "OTHER_PROJECT"
-        CREDENTIALS = _make_credentials()
-        client = self._make_one(project=PROJECT, credentials=CREDENTIALS)
+        project = "PROJECT"
+        other_project = "OTHER_PROJECT"
+        credentials = _make_credentials()
+        client = self._make_one(project=project, credentials=credentials)
 
-        BUCKET_NAME = "bucket-name"
+        bucket_name = "bucket-name"
         URI = "/".join(
             [
                 client._connection.API_BASE_URL,
                 "storage",
                 client._connection.API_VERSION,
-                "b?project=%s" % (OTHER_PROJECT,),
+                "b?project=%s" % (other_project,),
             ]
         )
         data = {"error": {"message": "Conflict"}}
-        sent = {"name": BUCKET_NAME}
+        json_expected = {"name": bucket_name}
         http = _make_requests_session(
             [_make_json_response(data, status=http_client.CONFLICT)]
         )
         client._http_internal = http
 
         with self.assertRaises(Conflict):
-            client.create_bucket(BUCKET_NAME, project=OTHER_PROJECT)
+            client.create_bucket(bucket_name, project=other_project)
 
         http.request.assert_called_once_with(
             method="POST", url=URI, data=mock.ANY, headers=mock.ANY
         )
         json_sent = http.request.call_args_list[0][1]["data"]
-        self.assertEqual(sent, json.loads(json_sent))
+        self.assertEqual(json_expected, json.loads(json_sent))
 
-    def test_create_bucket_success(self):
+    def test_create_bucket_with_object_conflict(self):
+        from google.cloud.exceptions import Conflict
         from google.cloud.storage.bucket import Bucket
 
-        PROJECT = "PROJECT"
-        CREDENTIALS = _make_credentials()
-        client = self._make_one(project=PROJECT, credentials=CREDENTIALS)
+        project = "PROJECT"
+        other_project = "OTHER_PROJECT"
+        credentials = _make_credentials()
+        client = self._make_one(project=project, credentials=credentials)
 
-        BUCKET_NAME = "bucket-name"
+        bucket_name = "bucket-name"
+        bucket_obj = Bucket(client, bucket_name)
         URI = "/".join(
             [
                 client._connection.API_BASE_URL,
                 "storage",
                 client._connection.API_VERSION,
-                "b?project=%s" % (PROJECT,),
+                "b?project=%s" % (other_project,),
             ]
         )
-        sent = {"name": BUCKET_NAME, "billing": {"requesterPays": True}}
-        data = sent
+        data = {"error": {"message": "Conflict"}}
+        http = _make_requests_session(
+            [_make_json_response(data, status=http_client.CONFLICT)]
+        )
+        client._http_internal = http
+
+        with self.assertRaises(Conflict):
+            client.create_bucket(bucket_obj, project=other_project)
+
+        http.request.assert_called_once_with(
+            method="POST", url=URI, data=mock.ANY, headers=mock.ANY
+        )
+        json_expected = {"name": bucket_name}
+        json_sent = http.request.call_args_list[0][1]["data"]
+        self.assertEqual(json_expected, json.loads(json_sent))
+
+    def test_create_bucket_with_string_success(self):
+        from google.cloud.storage.bucket import Bucket
+
+        project = "PROJECT"
+        credentials = _make_credentials()
+        client = self._make_one(project=project, credentials=credentials)
+
+        bucket_name = "bucket-name"
+        URI = "/".join(
+            [
+                client._connection.API_BASE_URL,
+                "storage",
+                client._connection.API_VERSION,
+                "b?project=%s" % (project,),
+            ]
+        )
+        json_expected = {"name": bucket_name, "billing": {"requesterPays": True}}
+        data = json_expected
         http = _make_requests_session([_make_json_response(data)])
         client._http_internal = http
 
-        bucket = client.create_bucket(BUCKET_NAME, requester_pays=True)
+        bucket = client.create_bucket(bucket_name, requester_pays=True)
 
         self.assertIsInstance(bucket, Bucket)
-        self.assertEqual(bucket.name, BUCKET_NAME)
+        self.assertEqual(bucket.name, bucket_name)
         self.assertTrue(bucket.requester_pays)
         http.request.assert_called_once_with(
             method="POST", url=URI, data=mock.ANY, headers=mock.ANY
         )
         json_sent = http.request.call_args_list[0][1]["data"]
-        self.assertEqual(sent, json.loads(json_sent))
+        self.assertEqual(json_expected, json.loads(json_sent))
+
+    def test_create_bucket_with_object_success(self):
+        from google.cloud.storage.bucket import Bucket
+
+        project = "PROJECT"
+        credentials = _make_credentials()
+        client = self._make_one(project=project, credentials=credentials)
+
+        bucket_name = "bucket-name"
+        bucket_obj = Bucket(client, bucket_name)
+        bucket_obj.storage_class = "COLDLINE"
+        bucket_obj.requester_pays = True
+
+        URI = "/".join(
+            [
+                client._connection.API_BASE_URL,
+                "storage",
+                client._connection.API_VERSION,
+                "b?project=%s" % (project,),
+            ]
+        )
+        json_expected = {
+            "name": bucket_name,
+            "billing": {"requesterPays": True},
+            "storageClass": "COLDLINE",
+        }
+        data = json_expected
+        http = _make_requests_session([_make_json_response(data)])
+        client._http_internal = http
+
+        bucket = client.create_bucket(bucket_obj)
+
+        self.assertIsInstance(bucket, Bucket)
+        self.assertEqual(bucket.name, bucket_name)
+        self.assertTrue(bucket.requester_pays)
+        http.request.assert_called_once_with(
+            method="POST", url=URI, data=mock.ANY, headers=mock.ANY
+        )
+        json_sent = http.request.call_args_list[0][1]["data"]
+        self.assertEqual(json_expected, json.loads(json_sent))
+
+    def test_download_blob_to_file_with_blob(self):
+        project = "PROJECT"
+        credentials = _make_credentials()
+        client = self._make_one(project=project, credentials=credentials)
+        blob = mock.Mock()
+        file_obj = io.BytesIO()
+
+        client.download_blob_to_file(blob, file_obj)
+        blob.download_to_file.assert_called_once_with(
+            file_obj, client=client, start=None, end=None
+        )
+
+    def test_download_blob_to_file_with_uri(self):
+        project = "PROJECT"
+        credentials = _make_credentials()
+        client = self._make_one(project=project, credentials=credentials)
+        blob = mock.Mock()
+        file_obj = io.BytesIO()
+
+        with mock.patch("google.cloud.storage.client.Blob", return_value=blob):
+            client.download_blob_to_file("gs://bucket_name/path/to/object", file_obj)
+
+        blob.download_to_file.assert_called_once_with(
+            file_obj, client=client, start=None, end=None
+        )
+
+    def test_download_blob_to_file_with_invalid_uri(self):
+        project = "PROJECT"
+        credentials = _make_credentials()
+        client = self._make_one(project=project, credentials=credentials)
+        blob = mock.Mock()
+        file_obj = io.BytesIO()
+
+        with mock.patch("google.cloud.storage.client.Blob", return_value=blob):
+            with pytest.raises(ValueError, match="URI scheme must be gs"):
+                client.download_blob_to_file(
+                    "http://bucket_name/path/to/object", file_obj
+                )
 
     def test_list_buckets_wo_project(self):
         CREDENTIALS = _make_credentials()
